import numpy as np
from pyTurbulence.solver import solve
from datetime import datetime
import os

def main():
    # Metadata
    code_name = "~~~ pyTurbulence: Synthetic Turbulence Generator ~~~"
    author = "Amaury Bilocq"
    date = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    # Parameters
    params = {
        "nbModes": 512,
        "Mt": 0.23,
        "Pressure": 1.0,
        "Temperature": 1.0,
        "k0": 12.0,
        "domain_size": (2. * np.pi, 2. * np.pi, 2. * np.pi),
        "domain_resolution": (64, 64, 64),
        "seed": 42,
        "gamma": 1.4,
        "case": 1,
        "Spectrum": "PassotPouquet"
    }

    # Create results directory
    results_dir = "results"
    if not os.path.exists(results_dir):
        os.makedirs(results_dir)

    # Run the solver
    results = solve(params, results_dir)

    # Log file
    log_file = os.path.join(results_dir, "log.txt")
    with open(log_file, "w") as f:
        # Display metadata
        f.write(f"{'='*70}\n")
        f.write(f"{code_name:^70}\n")
        f.write(f"{'='*70}\n")
        f.write(f"{'Author:':<20} {author}\n")
        f.write(f"{'Date:':<20} {date}\n")
        f.write(f"{'='*70}\n")

        # Display parameters
        f.write(f"{'Parameters':<20}\n")
        f.write(f"{'-'*70}\n")
        for key, value in params.items():
            f.write(f"{key:<20} : {value}\n")
        f.write(f"{'='*70}\n")

        # Display results
        f.write(f"{'Task':<50} {'Time (seconds)':>20}\n")
        f.write(f"{'-'*70}\n")
        f.write(f"{'Computing the solenoidal velocity field':<50} {results['solenoidal_time']:>20.4f}\n")
        f.write(f"{'Computing the incompressible pressure fluctuations':<50} {results['pressure_time']:>20.4f}\n")
        f.write(f"{'Computing the thermodynamic fields':<50} {results['thermodynamic_time']:>20.4f}\n")
        f.write(f"{'Plotting the spectrum':<50} {results['spectrum_time']:>20.4f}\n")
        f.write(f"{'Saving the data':<50} {results['save_time']:>20.4f}\n")
        f.write(f"{'-'*70}\n")
        f.write(f"{'Total TKE':<50} {results['TKE']:>20.4f}\n")
        f.write(f"{'Expected TKE':<50} {results['Ek']:>20.4f}\n")
        f.write(f"{'='*70}\n")

    # Print log to console
    with open(log_file, "r") as f:
        print(f.read())

if __name__ == "__main__":
    main()